@@ -21,6 +21,8 @@
 #define RSYNC_RSH_ENV "RSYNC_RSH"
 
 #define RSYNC_NAME "rsync"
+#define RSYNCD_CONF "/etc/rsyncd.conf"
+
 #define BACKUP_SUFFIX "~"
 
 /* a non-zero CHAR_OFFSET makes the rolling sum stronger, but is
@@ -43,15 +45,19 @@
 #define MIN_PROTOCOL_VERSION 11
 #define MAX_PROTOCOL_VERSION 30
 
+#define RSYNC_PORT 873
+
 #define SPARSE_WRITE_SIZE (1024)
 #define WRITE_SIZE (32*1024)
 #define CHUNK_SIZE (32*1024)
 #define MAX_MAP_SIZE (4*1024*1024)
 
+#define MAX_ARGS 1000
+
 #define BLOCKING_TIMEOUT 10
 
-#define FERROR stderr
-#define FINFO (am_server?stderr:stdout)
+#define FERROR 1
+#define FINFO 2
 
 #include "config.h"
 
@@ -66,6 +72,7 @@
 #include <unistd.h>
 #endif
 #include <stdio.h>
+#include <stddef.h>
 
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -163,10 +170,21 @@
 #include "lib/getopt.h"
 #endif
 
+#ifdef HAVE_GLOB
+#include <glob.h>
+#endif
+
 /* these are needed for the uid/gid mapping code */
 #include <pwd.h>
 #include <grp.h>
 
+#include <stdarg.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <syslog.h>
+#include <sys/file.h>
+
 #ifndef S_IFLNK
 #define S_IFLNK  0120000
 #endif
@@ -175,10 +193,18 @@
 #define S_ISLNK(mode) (((mode) & S_IFLNK) == S_IFLNK)
 #endif
 
+#define BOOL int
+
 #ifndef uchar
 #define uchar unsigned char
 #endif
 
+#if HAVE_UNSIGNED_CHAR
+#define schar signed char
+#else
+#define schar char
+#endif
+
 #ifndef int32
 #if (SIZEOF_INT == 4)
 #define int32 int
@@ -193,10 +219,25 @@
 #define uint32 unsigned int32
 #endif
 
-#ifdef HAVE_LONGLONG
+#if HAVE_OFF64_T
+#define OFF_T off64_t
+#define STRUCT_STAT struct stat64
+#else
+#define OFF_T off_t
+#define STRUCT_STAT struct stat
+#endif
+
+#if HAVE_OFF64_T
+#define int64 off64_t
+#elif (SIZEOF_LONG == 8) 
+#define int64 long
+#elif (SIZEOF_INT == 8) 
+#define int64 int
+#elif HAVE_LONGLONG
 #define int64 long long
 #else
 #define int64 off_t
+#define NO_INT64
 #endif
 
 #ifndef MIN
@@ -215,10 +256,14 @@
 #define MAXPATHLEN 1024
 #endif
 
+#ifndef INADDR_NONE
+#define INADDR_NONE 0xffffffff
+#endif
+
 struct file_struct {
 	unsigned flags;
 	time_t modtime;
-	off_t length;
+	OFF_T length;
 	mode_t mode;
 	ino_t inode;
 	dev_t dev;
@@ -239,7 +284,7 @@ struct file_list {
 };
 
 struct sum_buf {
-  off_t offset;			/* offset in file of this chunk */
+  OFF_T offset;			/* offset in file of this chunk */
   int len;			/* length of chunk of file */
   int i;			/* index of this chunk */
   uint32 sum1;	                /* simple checksum */
@@ -247,7 +292,7 @@ struct sum_buf {
 };
 
 struct sum_struct {
-  off_t flength;		/* total file length */
+  OFF_T flength;		/* total file length */
   int count;			/* how many chunks */
   int remainder;		/* flength % block_length */
   int n;			/* block_length */
@@ -257,7 +302,7 @@ struct sum_struct {
 struct map_struct {
 	char *map,*p;
 	int fd,p_size,p_len;
-	off_t size, p_offset;
+	OFF_T size, p_offset;
 };
 
 /* we need this function because of the silly way in which duplicate
@@ -269,22 +314,6 @@ static inline int flist_up(struct file_list *flist, int i)
 	return i;
 }
 
-
-#include "byteorder.h"
-#include "version.h"
-#include "proto.h"
-#include "md4.h"
-
-#if !HAVE_STRERROR
-extern char *sys_errlist[];
-#define strerror(i) sys_errlist[i]
-#endif
-
-#ifndef HAVE_STRCHR
-# define strchr                 index
-# define strrchr                rindex
-#endif
-
 #if HAVE_DIRENT_H
 # include <dirent.h>
 #else
@@ -300,6 +329,21 @@ extern char *sys_errlist[];
 # endif
 #endif
 
+#include "byteorder.h"
+#include "version.h"
+#include "proto.h"
+#include "md4.h"
+
+#if !HAVE_STRERROR
+extern char *sys_errlist[];
+#define strerror(i) sys_errlist[i]
+#endif
+
+#ifndef HAVE_STRCHR
+# define strchr                 index
+# define strrchr                rindex
+#endif
+
 #ifndef HAVE_ERRNO_DECL
 extern int errno;
 #endif
@@ -349,6 +393,22 @@ extern int errno;
 #define S_ISCHR(mode) (((mode) & (_S_IFMT)) == (_S_IFCHR))
 #endif
 
+#ifndef S_ISSOCK
+#ifdef _S_IFSOCK
+#define S_ISSOCK(mode) (((mode) & (_S_IFMT)) == (_S_IFSOCK))
+#else
+#define S_ISSOCK(mode) (0)
+#endif
+#endif
+
+#ifndef S_ISFIFO
+#ifdef _S_IFIFO
+#define S_ISFIFO(mode) (((mode) & (_S_IFMT)) == (_S_IFIFO))
+#else
+#define S_ISFIFO(mode) (0)
+#endif
+#endif
+
 #ifndef S_ISDIR
 #define S_ISDIR(mode) (((mode) & (_S_IFMT)) == (_S_IFDIR))
 #endif
@@ -357,5 +417,6 @@ extern int errno;
 #define S_ISREG(mode) (((mode) & (_S_IFMT)) == (_S_IFREG))
 #endif
 
-#define IS_DEVICE(mode) (S_ISCHR(mode) || S_ISBLK(mode))
+
+#define IS_DEVICE(mode) (S_ISCHR(mode) || S_ISBLK(mode) || S_ISSOCK(mode) || S_ISFIFO(mode))
 
